import {
  __spreadProps,
  __spreadValues,
  presets
} from "./chunk-HSDUPAKJ.mjs";

// src/index.ts
import { promises as fs } from "fs";
import { createUnplugin } from "unplugin";

// src/core/options.ts
import { resolve } from "path";
import { toArray } from "@antfu/utils";
import { createFilter } from "@rollup/pluginutils";
import hasPkg from "has-pkg";
function resolveOptions(options = {}) {
  const imports = flattenImportsMap(options.imports, options.presetOverriding);
  const { dts = hasPkg("typescript") } = options;
  const resolved = __spreadProps(__spreadValues({
    sourceMap: false
  }, options), {
    dts: dts === false ? false : dts === true ? resolve("auto-imports.d.ts") : resolve(dts),
    imports,
    matchRE: new RegExp(`\\b(${Object.keys(imports).join("|")})\\b`, "g"),
    idFilter: createFilter(options.include || [/\.[jt]sx?$/, /\.vue\??/, /\.svelte$/], options.exclude || [/node_modules/, /\.git/])
  });
  return resolved;
}
function flattenImportsMap(map, overriding = false) {
  const flat = {};
  toArray(map).forEach((definition) => {
    if (typeof definition === "string") {
      if (!presets[definition])
        throw new Error(`[auto-import] preset ${definition} not found`);
      const preset = presets[definition];
      definition = typeof preset === "function" ? preset() : preset;
    }
    for (const mod of Object.keys(definition)) {
      for (const id of definition[mod]) {
        const meta = {
          module: mod
        };
        if (Array.isArray(id)) {
          meta.name = id[1];
          meta.from = id[0];
        } else {
          meta.name = id;
        }
        if (flat[meta.name] && !overriding)
          throw new Error(`[auto-import] identifier ${meta.name} already defined with ${flat[meta.name].module}`);
        flat[meta.name] = meta;
      }
    }
  });
  return flat;
}

// src/core/transform.ts
import MagicString from "magic-string";
var excludeRE = [
  /\bimport\s*([\w_$]*?),?\s*\{([\s\S]*?)\}\s*from\b/g,
  /\bfunction\s*([\s\S]+?)\s*\(/g,
  /\b(?:const|let|var)\s*([\w\d_$]+?)\b/g
];
var multilineCommentsRE = /\/\*(.|[\r\n])*?\*\//gm;
var singlelineCommentsRE = /\/\/.*/g;
function stripeComments(code) {
  return code.replace(multilineCommentsRE, "").replace(singlelineCommentsRE, "");
}
function transform(code, id, { matchRE, imports, sourceMap }) {
  const noComments = stripeComments(code);
  const matched = new Set(Array.from(noComments.matchAll(matchRE)).map((i) => i[1]));
  if (!matched.size)
    return null;
  for (const regex of excludeRE) {
    Array.from(noComments.matchAll(regex)).flatMap((i) => {
      var _a, _b;
      return [...((_a = i[1]) == null ? void 0 : _a.split(",")) || [], ...((_b = i[2]) == null ? void 0 : _b.split(",")) || []];
    }).forEach((i) => matched.delete(i.trim()));
  }
  if (!matched.size)
    return null;
  const modules = {};
  Array.from(matched).forEach((name) => {
    var _a;
    const moduleName = (_a = imports[name]) == null ? void 0 : _a.module;
    if (!moduleName)
      return;
    if (!modules[moduleName])
      modules[moduleName] = [];
    modules[moduleName].push(imports[name]);
  });
  const importStatements = Object.entries(modules).map(([moduleName, names]) => {
    const imports2 = names.map(({ name, from }) => from ? `${from} as ${name}` : name).join(", ");
    return `import { ${imports2} } from '${moduleName}';`;
  }).join("");
  const s = new MagicString(code);
  s.prependLeft(0, importStatements);
  return {
    code: s.toString(),
    map: sourceMap ? s.generateMap() : null
  };
}

// src/core/dts.ts
function generateDeclration(imports) {
  const body = Object.entries(imports).map(([name, info]) => `  const ${name}: typeof import('${info.module}')['${info.from || name}']`).join("\n");
  return `// Generated by 'unplugin-auto-import'
declare global {
${body}
}
export {}
`;
}

// src/index.ts
var src_default = createUnplugin((options) => {
  const resolved = resolveOptions(options);
  if (!Object.keys(resolved.imports).length)
    console.warn("[auto-import] plugin installed but no imports has defined, see https://github.com/antfu/unplugin-auto-import#configurations for configurations");
  if (resolved.dts)
    fs.writeFile(resolved.dts, generateDeclration(resolved.imports), "utf-8");
  return {
    name: "unplugin-auto-import",
    enforce: "post",
    transformInclude(id) {
      return resolved.idFilter(id);
    },
    transform(code, id) {
      return transform(code, id, resolved);
    }
  };
});

export {
  src_default
};
